<?php

// Abstract class representing a purchasable item
abstract class Purchasable {
    abstract public function calculateTotalPrice();
}

// Trait for discountable items
trait Discountable {
    protected $discount = 0;

    public function applyDiscount($amount) {
        $this->discount = $amount;
    }

    public function getDiscount() {
        return $this->discount;
    }
}

// Base class representing a generic product
class Product extends Purchasable {
    protected $id;
    protected $name;
    protected $price;
    protected $quantity;

    public function __construct($id, $name, $price, $quantity) {
        $this->id = $id;
        $this->name = $name;
        $this->price = $price;
        $this->quantity = $quantity;
    }

    public function getId() {
        return $this->id;
    }

    public function getName() {
        return $this->name;
    }

    public function getPrice() {
        return $this->price;
    }

    public function getQuantity() {
        return $this->quantity;
    }

    public function calculateTotalPrice() {
        return $this->price * $this->quantity;
    }
}

// Derived class for electronic products
class ElectronicProduct extends Product {
    protected $brand;
    protected $warranty;

    public function __construct($id, $name, $price, $quantity, $brand, $warranty) {
        parent::__construct($id, $name, $price, $quantity);
        $this->brand = $brand;
        $this->warranty = $warranty;
    }

    public function getBrand() {
        return $this->brand;
    }

    public function getWarranty() {
        return $this->warranty;
    }

    public function displayInfo() {
        echo "Electronic Product: {$this->getName()} - {$this->getBrand()} - {$this->getPrice()} - {$this->getQuantity()} - {$this->getWarranty()}";
    }
}

// Derived class for clothing products
class ClothingProduct extends Product {
    protected $size;
    protected $material;

    public function __construct($id, $name, $price, $quantity, $size, $material) {
        parent::__construct($id, $name, $price, $quantity);
        $this->size = $size;
        $this->material = $material;
    }

    public function getSize() {
        return $this->size;
    }

    public function getMaterial() {
        return $this->material;
    }

    public function displayInfo() {
        echo "Clothing Product: {$this->getName()} - {$this->getSize()} - {$this->getMaterial()} - {$this->getPrice()} - {$this->getQuantity()}";
    }
}

// Example usage
$electronicProduct = new ElectronicProduct('E001', 'Laptop', 999, 2, 'Dell', '1 year');
$clothingProduct = new ClothingProduct('C001', 'T-Shirt', 20, 5, 'M', 'Cotton');

$electronicProduct->displayInfo();
echo "\n";
$clothingProduct->displayInfo();
echo "\n";

// Applying discount
$electronicProduct->applyDiscount(50);
echo "Discount applied to electronic product: {$electronicProduct->getDiscount()}";
